/*
 * umaxoffd.c - a little daemon to turn off the lamp on the Umax
 * 1220U scanner after a period of inactivity (default 5 minutes).
 *
 * Paul Mackerras, December 1999
 *
 * Copyright 1999 Paul Mackerras.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>

int fd;

struct cmsg_ioctl {
	unsigned char	requesttype;
	unsigned char	request;
	unsigned char	value[2];
	unsigned char	index[2];
	unsigned char	length[2];
	void		*data;
};

void control_msg(int rtype, int req, int value, int index, int len,
		 void *data)
{
	struct cmsg_ioctl c;

	c.requesttype = rtype;
	c.request = req;
	c.value[0] = value;
	c.value[1] = value >> 8;
	c.index[0] = index;
	c.index[1] = index >> 8;
	c.length[0] = len;
	c.length[1] = len >> 8;
	c.data = data;
	if (ioctl(fd, 0x556d, &c) < 0) {
		perror("scanner control");
		exit(1);
	}
}

int getstat(int which)
{
	unsigned char buf[1];

	control_msg(0xc0, 0, 0, which, 1, buf);
	return buf[0];
}

int xpectstat(int which, int val)
{
	int s = getstat(which);
	return s == val;
}

int waitstat(int which, int val)
{
	int s, n = 0;

	for (;;) {
		s = getstat(which);
		if (s == val)
			return 1;
		if (++n >= 100)
			return 0;
		usleep(100000);
	}
}

void xxxops(void)
{
	control_msg(0x40, 1, 2, 3, 0, 0);
	control_msg(0x40, 1, 14, 2, 0, 0);
	control_msg(0x40, 1, 64, 0, 0, 0);
	control_msg(0x40, 1, 6, 2, 0, 0);
	xpectstat(4, 0x38);
	control_msg(0x40, 1, 7, 2, 0, 0);
	xpectstat(4, 0x38);
	control_msg(0x40, 1, 4, 2, 0, 0);
	xpectstat(4, 0xf8);
	control_msg(0x40, 1, 5, 2, 0, 0);
	xpectstat(2, 5);
	control_msg(0x40, 1, 4, 2, 0, 0);
	control_msg(0x40, 1, 30, 3, 0, 0);
}

int send(unsigned char *buf, int nb)
{
	int ret;

	control_msg(0x40, 4, 0, 0, 0, 0);
	control_msg(0x40, 3, nb, 0, 0, 0);
	ret = write(fd, buf, nb);
	if (ret < 0) {
		perror("write");
		return 0;
	}
	return 1;
}

int usync(int len, int cmd)
{
	unsigned char buf[4];
	int s0, s4;

	buf[0] = 0x55;
	buf[1] = 0xaa;
	if (!send(buf, 2))
		return 0;
	if (!waitstat(4, 0xc8))
		return 0;
	buf[0] = len >> 16;
	buf[1] = len >> 8;
	buf[2] = len;
	buf[3] = cmd;
	if (!send(buf, 4))
		return 0;
	s0 = getstat(0);
	s4 = getstat(4);
	return 1;
}

int qsend(unsigned char *buf, int nb)
{
	int s0, s4, ret;

	if (!waitstat(4, 0xc8))
		return 0;
	if (!send(buf, nb))
		return 0;
	s4 = getstat(4);
	s0 = getstat(0);
	return 1;
}

unsigned char opc[16] = {
	2, 0x80, 0, 0x70, 0, 0, 0x60, 0x2f, 0x2f, 7, 0, 0, 0, 0x80, 0x90, 0
};

main(int ac, char **av)
{
	unsigned long t, offt;
	struct timeval tnow;
	int idle_limit, ts;
	struct stat sbuf;

	idle_limit = 0;
	if (ac > 1)
		idle_limit = atoi(av[1]);
	if (idle_limit <= 0)
		idle_limit = 300;

	offt = 0;
	ts = 0;
	for (;;) {
		if (ts > 0) {
			if (ts > 60)
				ts = 60;
			sleep(ts);
		}
		ts = idle_limit;
		if (stat("/dev/usbscanner", &sbuf) < 0)
			continue;
		t = sbuf.st_atime;
		if (t <= offt)
			continue;	/* lamp is already off */
		gettimeofday(&tnow, NULL);
		if (tnow.tv_sec >= t + idle_limit) {
			fd = open("/dev/usbscanner", O_RDWR | O_EXCL);
			if (fd < 0)
				continue;	/* not plugged in or in use */
			usync(0, 0);
			xxxops();
			usync(0, 0);
			usync(16, 0x82);
			qsend(opc, 16);
			close(fd);
			gettimeofday(&tnow, NULL);
			offt = t = tnow.tv_sec;
		}
		ts = t + idle_limit - tnow.tv_sec;
	}
	exit(0);
}
